package net.sf.itcb.common.portlet.vaadin;

import java.io.Serializable;
import java.util.Map;

import javax.portlet.PortletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sf.itcb.common.portlet.vaadin.page.AbstractItcbPortletPage;

import com.vaadin.ui.Panel;

public class PageMappingProcessor implements Serializable {

	private static final long serialVersionUID = 1L;
	
	protected final Logger log = LoggerFactory.getLogger(getClass());
	
	private Map<String, AbstractItcbPortletPage> mapping;
	
	private String defaultPageKey;

	private Panel panel;
	
	private AbstractItcbPortletPage currentPage;
	
	private PortletRequest request;
	
	
	/**
	 * Define the pages mapping
	 * Has to be set by Spring context at creation
	 * @param mapping mapping that will be used for reach a page
	 */
	public void setMapping(Map<String, AbstractItcbPortletPage> mapping) {
		this.mapping = mapping;
	}
	

	/**
	 * Define the default page. This key has to be in mapping list
	 * Has to be set by Spring context at creation
	 * @param defaultPage the default page to show
	 */
	public void setDefaultPageKey(String defaultPageKey) {
		this.defaultPageKey = defaultPageKey;
	}
	
	/**
	 * The request is set by the application at the initialisation. It also allows to deal with session.
	 * @param request
	 */
	public void setRequest(PortletRequest request) {
		this.request = request;
	}
	
	/**
	 * @param componentContainer componentContainer to be set by the application : In most cases the componentContainer is the main window
	 * It's the componentContainer that has to be erased and refill by content
	 */
	public void setPanel(Panel panel) {
		this.panel = panel;
	}
	
	/**
	 * Display the page associated with the id refPage. The page is simply displayed as it was the last time it has been displayed for this session
	 * If it is the first time, the page is initialized.
	 * If the <code>reload</code> parameter is set to <code>true</code>, page content is reloaded 
	 * @param refPage the page id referenced in <code>mapping</code>
	 * @param reload defines if the page has to be reloaded or not
	 */
	public void displayPage(String refPage, boolean reload) {
		AbstractItcbPortletPage page = mapping.get(refPage);	
		if(currentPage != null && currentPage != page) {
			currentPage.setVisible(false);
		}
		if(page != null && reload) {
			page.removeAllComponents();
			page.initPage();
		}
		if(page.getPageMappingProcessor() == null || page.getParent() == null || !page.getComponentIterator().hasNext()) {
			page.setPageMappingProcessor(this);
			page.initPage();
			panel.addComponent(page);
		}
		page.setVisible(true);
		currentPage=page;
	}
	
	
	/**
	 * Display the default page defined in Spring context for the mapping processor
	 */
	public void displayDefaultPage() {
		if(defaultPageKey != null) {
			panel.removeAllComponents();
			displayPage(defaultPageKey, false);
		}
		else {
			if(log.isDebugEnabled()) {
				log.warn("No default page configured. PageMappingProcessor mecanism not used. Set the defaultPageKey property in your spring context.");
			}
		}
	}
	
	/**
	 * Store an attribute in session
	 * @param key
	 * @param value
	 */
	public void setSessionAttribute (String key, Object value) {
		request.getPortletSession().setAttribute(key, value);
	}
	
	/**
	 * Get an object from his key in session 
	 * @param key
	 * @param requiredType
	 * @return 
	 */
	public <T> T getSessionAttribute(String key, Class<T> requiredType) {
		
		return (T) request.getPortletSession().getAttribute(key);
	}
	
}
